Attribute VB_Name = "RegEdit"
'---------------------------------------------------------------------------'
'----- Module for manipulacting with RegEdit's Keys and Values -----'
'----- by Marko Treechko Trichkovich -----'
'----- t.marko016@gmail.com -----'
'----- http://www.hi-technology.tk -----'
'---------------------------------------------------------------------------'

Option Explicit

'-------------------------
'----- API Functions -----
'-------------------------
Private Declare Function RegCloseKey Lib "advapi32" (ByVal hKey As Long) As Long
Private Declare Function RegCreateKeyEx Lib "advapi32" Alias "RegCreateKeyExA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal Reserved As Long, ByVal lpClass As String, ByVal dwOptions As Long, ByVal samDesired As Long, ByRef lpSecurityAttributes As SECURITY_ATTRIBUTES, ByRef phkResult As Long, ByRef lpdwDisposition As Long) As Long
Private Declare Function RegOpenKeyEx Lib "advapi32" Alias "RegOpenKeyExA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal ulOptions As Long, ByVal samDesired As Long, ByRef phkResult As Long) As Long
Private Declare Function RegQueryValueEx Lib "advapi32" Alias "RegQueryValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal lpReserved As Long, ByRef lpType As Long, ByVal lpData As String, ByRef lpcbData As Long) As Long
Private Declare Function RegSetValueEx Lib "advapi32" Alias "RegSetValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal Reserved As Long, ByVal dwType As Long, ByVal lpData As String, ByVal cbData As Long) As Long
Private Declare Function RegDeleteValue Lib "advapi32.dll" Alias "RegDeleteValueA" (ByVal hKey As Long, ByVal lpValueName As String) As Long
Private Declare Function RegDeleteKey Lib "advapi32.dll" Alias "RegDeleteKeyA" (ByVal hKey As Long, ByVal lpSubKey As String) As Long
Private Declare Function RegQueryInfoKey Lib "advapi32.dll" Alias "RegQueryInfoKeyA" (ByVal hKey As Long, ByVal lpClass As String, lpcbClass As Long, ByVal lpReserved As Long, lpcSubKeys As Long, lpcbMaxSubKeyLen As Long, lpcbMaxClassLen As Long, lpcValues As Long, lpcbMaxValueNameLen As Long, lpcbMaxValueLen As Long, lpcbSecurityDescriptor As Long, lpftLastWriteTime As FILETIME) As Long
Private Declare Function RegEnumKeyEx Lib "advapi32.dll" Alias "RegEnumKeyExA" (ByVal hKey As Long, ByVal dwIndex As Long, ByVal lpName As String, lpcbName As Long, ByVal lpReserved As Long, ByVal lpClass As String, lpcbClass As Long, lpftLastWriteTime As FILETIME) As Long

'--------------------------
'----- Reg Data Types -----
'--------------------------
Public Const REG_SZ = 1
Public Const REG_EXPAND_SZ = 2
Public Const REG_BINARY = 3
Public Const REG_DWORD = 4

'----------------------------------
'----- Reg Create Type Values -----
'----------------------------------
Private Const REG_OPTION_NON_VOLATILE = 0       ' Key is preserved when system is rebooted

'------------------------------------
'----- Reg Key Security Options -----
'------------------------------------
Private Const READ_CONTROL = &H20000
Private Const KEY_QUERY_VALUE = &H1
Private Const KEY_SET_VALUE = &H2
Private Const KEY_CREATE_SUB_KEY = &H4
Private Const KEY_ENUMERATE_SUB_KEYS = &H8
Private Const KEY_NOTIFY = &H10
Private Const KEY_CREATE_LINK = &H20
Private Const KEY_READ = KEY_QUERY_VALUE + KEY_ENUMERATE_SUB_KEYS + KEY_NOTIFY + READ_CONTROL
Private Const KEY_WRITE = KEY_SET_VALUE + KEY_CREATE_SUB_KEY + READ_CONTROL
Private Const KEY_EXECUTE = KEY_READ
Private Const KEY_ALL_ACCESS = KEY_QUERY_VALUE + KEY_SET_VALUE + _
                       KEY_CREATE_SUB_KEY + KEY_ENUMERATE_SUB_KEYS + _
                       KEY_NOTIFY + KEY_CREATE_LINK + READ_CONTROL
                     
'------------------------------
'----- Reg Key ROOT Types -----
'------------------------------
Public Const HKEY_CLASSES_ROOT = &H80000000
Public Const HKEY_CURRENT_USER = &H80000001
Public Const HKEY_LOCAL_MACHINE = &H80000002
Public Const HKEY_USERS = &H80000003
Public Const HKEY_PERFORMANCE_DATA = &H80000004

'------------------------
'----- Return Value -----
'------------------------
Private Const ERROR_SUCCESS = 0&
Private Const ERROR_FILE_NOT_FOUND = 2&         ' Registry path does not exist
Private Const ERROR_ACCESS_DENIED = 5&          ' Requested permissions not available
Private Const ERROR_INVALID_HANDLE = 6&         ' Invalid handle or top-level key
Private Const ERROR_BAD_NETPATH = 53            ' Network path not found
Private Const ERROR_INVALID_PARAMETER = 87      ' Bad parameter to a Win32 API function
Private Const ERROR_CALL_NOT_IMPLEMENTED = 120& ' Function valid only in WinNT/2000?XP
Private Const ERROR_INSUFFICIENT_BUFFER = 122   ' Buffer too small to hold data
Private Const ERROR_BAD_PATHNAME = 161          ' Registry path does not exist
Private Const ERROR_MORE_DATA = 234             '
Private Const ERROR_NO_MORE_ITEMS = 259&        ' Invalid enumerated value
Private Const ERROR_BADDB = 1009                ' Corrupted registry
Private Const ERROR_BADKEY = 1010               ' Invalid registry key
Private Const ERROR_CANTOPEN = 1011&            ' Cannot open registry key
Private Const ERROR_CANTREAD = 1012&            ' Cannot read from registry key
Private Const ERROR_CANTWRITE = 1013&           ' Cannot write to registry key
Private Const ERROR_REGISTRY_RECOVERED = 1014&  ' Recovery of part of registry successful
Private Const ERROR_REGISTRY_CORRUPT = 1015&    ' Corrupted registry
Private Const ERROR_REGISTRY_IO_FAILED = 1016&  ' Input/output operation failed
Private Const ERROR_NOT_REGISTRY_FILE = 1017&   ' Input file not in registry file format
Private Const ERROR_KEY_DELETED = 1018&         ' Key already deleted
Private Const ERROR_KEY_HAS_CHILDREN = 1020&    ' Key has subkeys & cannot be deleted

'---------------------------------------------
'----- Registry Security Attributes TYPE -----
'---------------------------------------------
Private Type SECURITY_ATTRIBUTES
    nLength As Long
    lpSecurityDescriptor As Long
    bInheritHandle As Boolean
End Type

Private Type FILETIME
    dwLowDateTime As Long
    dwHighDateTime As Long
End Type


'------------------------------------------------------------------------------'
'----- This function saves settings into Registry Editor on declared Path. ----'
'------------------------------------------------------------------------------'
Public Sub CreateSetting(ByVal hKey As Long, ByVal sPath As String, ByVal sValueName As String, ByVal sStringBuffer As String, ByVal lValueType As Long)

    Dim hRegKey As Long                              '----- Handle for opened key -----'
    Dim lRetVal As Long                              '----- Returned value from API's -----'
    Dim lNewOrUsed As Long                           '----- New key or alredy exist -----'
    Dim tSecurityAttributes As SECURITY_ATTRIBUTES   '----- Default security premission -----'

    '----- Open the key -----'
    If RegOpenKeyEx(hKey, sPath, 0, KEY_ALL_ACCESS, hRegKey) = ERROR_SUCCESS Or _
        RegOpenKeyEx(hKey, sPath, 0, KEY_ALL_ACCESS, hRegKey) = ERROR_FILE_NOT_FOUND Then

        '----- Size of the structure -----'
        tSecurityAttributes.nLength = Len(tSecurityAttributes)

        '----- Default security level -----'
        tSecurityAttributes.lpSecurityDescriptor = 0

        '----- The default value for this setting -----'
        tSecurityAttributes.bInheritHandle = True
        
        '----- Create or open the registry key -----'
        lRetVal = RegCreateKeyEx(hKey, sPath, 0, lValueType, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, tSecurityAttributes, hRegKey, lNewOrUsed)

        '----- Error during opening RegKey -----'
        If lRetVal <> ERROR_SUCCESS Then GoTo OpeningError

        '----- Ask which ValueType is currently in Use -----'
        If lValueType = REG_SZ Then

            '----- Note how a null character must be appended to the string -----'
            sStringBuffer = sStringBuffer & vbNullChar
            
            '----- Write the string -----'
            lRetVal = RegSetValueEx(hRegKey, sValueName, 0, REG_SZ, sStringBuffer, Len(sStringBuffer))
            
            '----- Error during Write RegKey -----'
            If lRetVal <> ERROR_SUCCESS Then GoTo WritingError
        
        '----- ElseIf sValueType = "Binary" Then -----'
        ElseIf lValueType = REG_BINARY Then
           
            '----- Write Binary value in Registry -----'
            lRetVal = RegSetValueEx(hRegKey, sValueName, 0, REG_BINARY, sStringBuffer, Len(sStringBuffer))
            
            '----- Error during Write RegKey -----'
            If lRetVal <> ERROR_SUCCESS Then GoTo WritingError

        '----- Loop for DWORD Value -----'
        ElseIf lValueType = REG_DWORD Then
        
            Dim lVal As Long
            lVal = 1
    
            '----- Write Hex value in Registry -----'
            lRetVal = RegSetValueEx(hRegKey, sValueName, 0, REG_DWORD, lVal, Len(lVal))

            '----- Error during Write RegKey -----'
            If lRetVal <> ERROR_SUCCESS Then GoTo WritingError
        End If

        '----- Close the registry key -----'
        RegCloseKey (hRegKey)
    Else
        '----- Show error message, something is wrong with the key -----'
        GoTo OpeningError
    End If

Exit Sub

'----- If something goes wrong while opening the key then show this message. -----'
OpeningError:
MsgBox "Error opening or creating registry key." & vbNewLine & _
"Error Number: " & lRetVal & vbNewLine & vbNewLine & _
"Press OK to abort.", vbExclamation, App.ProductName & ": Registry Key Access Error"
End

'----- If something goes wrong while writing value in the key then show this message -----'
WritingError:
MsgBox "Unable to write in registry." & vbNewLine & _
"Press OK to abort.", vbExclamation, App.ProductName & ": Registry Key Writing Error"
End
End Sub

'------------------------------------------------------------'
'----- Function returns key value from Registry Editor. -----'
'------------------------------------------------------------'
Public Function LoadSetting(ByVal hKey As Long, ByVal sPath As String, ByVal sValueName As String) As String

    Dim hRegKey As Long                             '----- Handle for opened key -----'
    Dim lRetVal As Long                             '----- Returned value from API's -----'
    Dim lValueType As Long                          '----- Recive ValueType from API's -----'
    Dim sBuff As String                             '-----
    Dim lDataBuffSize As Long                       '----- Lenght size of sBuff = Received String -----'

    '----- Open the key -----'
    If RegOpenKeyEx(hKey, sPath, 0, KEY_ALL_ACCESS, hRegKey) = ERROR_SUCCESS Or _
        RegOpenKeyEx(hKey, sPath, 0, KEY_ALL_ACCESS, hRegKey) = ERROR_FILE_NOT_FOUND Then
    
        '----- Retrieve information about the key -----'
        lRetVal = RegQueryValueEx(hRegKey, sValueName, 0, lValueType, "", lDataBuffSize)

        '----- Check if is ValueType = String  -----'
        If lValueType = REG_SZ Then

            '----- Check for ERROR_MORE_DATA -----'
            If lRetVal = ERROR_MORE_DATA Then
                
                '----- Get String Size -----'
                sBuff = Space(lDataBuffSize)
                
                '----- Retrieve the key's content -----'
                lRetVal = RegQueryValueEx(hRegKey, sValueName, 0, REG_SZ, ByVal sBuff, lDataBuffSize)

                '----- Error during Reading Key Value -----'
                If lRetVal <> ERROR_SUCCESS Then GoTo ReadingError
            End If
            
            '-----Remove null character -----
            If lRetVal = ERROR_SUCCESS Then LoadSetting = Left$(sBuff, lDataBuffSize - 1)
        
        '----- Check if is ValueType = Binary -----'
        ElseIf lValueType = REG_BINARY Then
            
            Dim sData As Boolean
            '----- Retrieve the key's value -----'
            lDataBuffSize = Len(sData)

            '----- Retrieve the key's content -----'
            lRetVal = RegQueryValueEx(hRegKey, sValueName, 0, REG_BINARY, sData, lDataBuffSize)

            '----- Error during Read Key Value -----'
            If lRetVal <> ERROR_SUCCESS Then GoTo ReadingError

            '----- Set value to the function -----'
            If lRetVal = ERROR_SUCCESS Then LoadSetting = sData

        End If
        
        '----- Close the Key -----'
        RegCloseKey (hRegKey)
    Else
        '----- Show error message, something is wrong with the key -----'
        GoTo OpeningError:
    End If

Exit Function

'----- If something goes wrong while opening the key then show this message. -----'
OpeningError:
MsgBox "Error opening or creating registry key." & vbNewLine & _
"Error Number: " & lRetVal & vbNewLine & vbNewLine & _
"Press OK to abort.", vbExclamation, App.ProductName & ": Registry Key Access Error"
End

'----- If something goes wrong while reading the key then show this message. -----'
ReadingError:
MsgBox "Error reading value from registry key." & vbNewLine & _
"Error Number: " & lRetVal & vbNewLine & vbNewLine & _
"Press OK to abort.", vbExclamation, App.ProductName & ": Registry Key Access Error"
End
End Function

'-----------------------------------------------------------------'
'----- Function returns number of SubKeys from declared Path -----'
'-----------------------------------------------------------------'
Public Function NumOfKeys(ByVal hKey As Long, ByVal sPath As String) As Integer

    Dim hRegKey As Long             '----- Handle for opened key -----'
    Dim lRetVal As Long             '----- Returned value from API's -----'
    Dim lNumKeys As Long            '----- Number of subkeys of open key -----'
    Dim lSubkeyNameLen As Long      '----- Length of the longest subkey name -----'
    Dim ft As FILETIME
    Dim Sa As SECURITY_ATTRIBUTES    '----- Use default security attributes of process -----'
    Sa.nLength = Len(Sa)
    Sa.bInheritHandle = CLng(True)

    '----- Open the key -----'
    If RegOpenKeyEx(hKey, sPath, 0, KEY_READ, hRegKey) = ERROR_SUCCESS Then
   
        '----- Get subkey information ------'
        lRetVal = RegQueryInfoKey(hRegKey, vbNullString, 0, 0, lNumKeys, lSubkeyNameLen, _
                                                                        0, 0, 0, 0, 0, ft)
        '----- Return value to the function if is there no error -----'
        If lRetVal = ERROR_SUCCESS Then NumOfKeys = lNumKeys

        '----- Close Key -----'
        RegCloseKey (hRegKey)
    Else
        '----- Show error message, something is wrong with the key -----'
        GoTo OpeningError
    End If

Exit Function

'----- If something goes wrong while opening the key then show this message. -----'
OpeningError:
MsgBox "Error opening or creating registry key." & vbNewLine & _
"Error Number: " & lRetVal & vbNewLine & vbNewLine & _
"Press OK to abort.", vbExclamation, App.ProductName & ": Registry Key Access Error"
End
End Function

'------------------------------------------------------------------------------'
'----- Function returns boolean value if key exist or not (True or False) -----'
'------------------------------------------------------------------------------'
Public Function isRegKeyExist(ByVal hKey As Long, ByVal sPath As String) As Boolean

    Dim hRegKey As Long             '----- Handle for opened key -----'
    Dim lRetVal As Long             '----- Returned value from API's -----'
    
    '----- Open the key -----'
    lRetVal = RegOpenKeyEx(hKey, sPath, 0, KEY_READ, hRegKey)
    
    '----- Checking for errors -----'
    If lRetVal = ERROR_SUCCESS Then

        '----- Return value to the function -----'
        isRegKeyExist = True

    ElseIf lRetVal = ERROR_FILE_NOT_FOUND Then
        
        '----- Return value to the function -----'
        isRegKeyExist = False

    Else
        GoTo OpeningError
    End If

    '----- Close RegKey -----'
    RegCloseKey (hRegKey)

Exit Function

'----- If something goes wrong while opening the key then show this message. -----'
OpeningError:
MsgBox "Error opening or creating registry key." & vbNewLine & _
"Error Number: " & lRetVal & vbNewLine & vbNewLine & _
"Press OK to abort.", vbExclamation, App.ProductName & ": Registry Key Access Error"
End
End Function

Public Sub DeleteKey(ByVal hKey As Long, sSubKey As Long, ByVal sValueName As String)
    Dim lResult As Long
    
    '------------------------'
    '----- Open the Key -----'
    '------------------------'
    If RegOpenKeyEx(hKey, sSubKey, 0, KEY_ALL_ACCESS, lResult) = ERROR_SUCCESS Then
    
        '--------------------------'
        '----- Delete the Key -----'
        '--------------------------'
        lResult = RegDeleteKey(lResult, vbNullString)

        '-------------------------'
        '----- Close the Key -----'
        '-------------------------'
        RegCloseKey (lResult)
    
    Else
        Exit Sub
    End If
End Sub
